//iobuddy.h
//XInput-to-JVS converter for NAOMI (common defines)
//Bryan E. Topp <betopp@betopp.com> 2018

//This file is part of IO-Buddy. (2018-09-30)

//IO-Buddy is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//IO-Buddy is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with IO-Buddy.  If not, see <https://www.gnu.org/licenses/>.

#pragma once
#include <stdint.h>
#include <xc.h>

#define SYS_FREQ 		(48000000UL) //48MHz frequency basically everywhere
#define _XTAL_FREQ      (48000000UL)


#define CPU_CLOCK_HZ             (48000000UL)    // CPU Clock Speed in Hz
#define CPU_CT_HZ            (CPU_CLOCK_HZ/2)    // CPU CoreTimer   in Hz
#define PERIPHERAL_CLOCK_HZ      (48000000UL)    // Peripheral Bus  in Hz

#define US_TO_CT_TICKS  (CPU_CT_HZ/1000000UL)    // uS to CoreTimer Ticks 

static void ShortDelay(                       // Short Delay
  uint32_t DelayCount)                   // Delay Time (CoreTimer Ticks)
{
  uint32_t StartTime;                    // Start Time
  StartTime =  ReadCoreTimer();        // Get CoreTimer value for StartTime
  while ( (uint32_t)( ReadCoreTimer() - StartTime) < DelayCount ) {};
} 


static void DelayMS(uint32_t msec)
{
    ShortDelay( msec * 1000 * US_TO_CT_TICKS );
}

static void DelayUS(uint32_t usec)
{
    ShortDelay( usec * US_TO_CT_TICKS );
}

static void SWReset()
{
    /* The following code illustrates a software Reset */
    // assume interrupts are disabled
    // assume the DMA controller is suspended
    // assume the device is locked
    /* perform a system unlock sequence */
    // starting critical sequence
    SYSKEY = 0x00000000; //write invalid key to force lock
    SYSKEY = 0xAA996655; //write key1 to SYSKEY
    SYSKEY = 0x556699AA; //write key2 to SYSKEY
    // OSCCON is now unlocked
    /* set SWRST bit to arm reset */
    RSWRSTSET = 1;
    /* read RSWRST register to trigger reset */
    unsigned int dummy;
    dummy = RSWRST;
    /* prevent any unwanted code execution until reset occurs*/
    while(1);
}