//jvsio.h
//JVS I/O bus protocol
//Bryan E. Topp <betopp@betopp.com> 2018

//This file is part of IO-Buddy. (2018-09-30)

//IO-Buddy is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//IO-Buddy is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with IO-Buddy.  If not, see <https://www.gnu.org/licenses/>.

#pragma once
#include <stdint.h>

//Initializes the JVS IO bus.
void JVSInit();

//Updates our communications on the bus; sends/receives incrementally. Responds to packets.
void JVSUpdate();

//Sets the control bits for response. These are used to build responses.
void JVSSetPlayerBits(uint8_t plidx, uint16_t bitmask);

//Sets the system bits for response.
void JVSSetSystemBits(uint8_t bitmask);

//Sets an analog axis for response - Unsigned 16-bit centered at 0x8000
void JVSSetAnalogValue(uint8_t axidx, uint16_t value);

//JVS bitmasks - we store 16 bits of relevant player input
enum JVSPlayerBits_e
{
	//First byte of response is tilt switches and reserved fields.
	
	//Second byte starts actual controls
	JVSPB_PUSH2  = 0x0001,
	JVSPB_PUSH1  = 0x0002,
	JVSPB_RIGHT  = 0x0004,
	JVSPB_LEFT   = 0x0008,
	JVSPB_DOWN   = 0x0010,
	JVSPB_UP     = 0x0020,
	JVSPB_SVC    = 0x0040,
	JVSPB_START  = 0x0080,
	
	//Third byte is more controls
	JVSPB_PUSH10 = 0x0100, //9 and 10 aren't actually defined,
	JVSPB_PUSH9  = 0x0200, //but there's empty space after 8
	JVSPB_PUSH8  = 0x0400,
	JVSPB_PUSH7  = 0x0800,
	JVSPB_PUSH6  = 0x1000,
	JVSPB_PUSH5  = 0x2000,
	JVSPB_PUSH4  = 0x4000,
	JVSPB_PUSH3  = 0x8000	
};

//JVS bitmasks - system switches
enum JVSSystemBits_e
{
    JVSSB_TEST       = 0x80,
    JVSSB_TILT1      = 0x40,
    JVSSB_TILT2      = 0x20,
    JVSSB_TILT3      = 0x10,
    JVSSB_RESERVED_1 = 0x08,
    JVSSB_RESERVED_2 = 0x04,
    JVSSB_RESERVED_3 = 0x02,
    JVSSB_RESERVED_4 = 0x01
};