//usbhost.h
//Functions for hosting the USB side of the iobuddy
//Bryan E. Topp <betopp@betopp.com> 2018

//This file is part of IO-Buddy. (2018-09-30)

//IO-Buddy is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//IO-Buddy is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with IO-Buddy.  If not, see <https://www.gnu.org/licenses/>.

#pragma once
#include <stdint.h>

//Initializes USB host and enumerates devices
void USBInit();

//Polls device for latest controls
void USBUpdate();

//Returns the VID of the player's pad, or 0 if no pad
uint16_t USBGetPadVID(uint8_t player);

//Returns the PID of the player's pad, or 0 if no pad
uint16_t USBGetPadPID(uint8_t player);

//Gets the latest action buttons from a player's controller
//(No real order, because it differs on each controller. Rebind it!)
uint8_t USBGetActionButtons(uint8_t player);

//Gets the latest directional controls from a player's controller
uint8_t USBGetDirectionButtons(uint8_t player);
#define POLLED_DIR_UP 0x01
#define POLLED_DIR_DOWN 0x02
#define POLLED_DIR_LEFT 0x04
#define POLLED_DIR_RIGHT 0x08

//Returns the "system" buttons state from the player's controller
uint8_t USBGetSystemButtons(uint8_t player);
#define POLLED_SYS_START 0x10 //Start/+/Options
#define POLLED_SYS_SELECT 0x20 //Select/Back/-/Share
#define POLLED_SYS_HOME 0x40 //Xbox/PlayStation/Home

//Analog hackery - returns analog controls
//No decent way to remap this stuff yet...
//JVS wants an unsigned 16-bit value
uint16_t USBGetAnalogAxis(uint8_t player, uint8_t axis);